#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.pring import pring
from pixy.colours import foreground, background
from pixy import decorators
from pixy.Font import Font
from pixy.EscapeSequence import EscapeSequence
from pixy.ExtendedColour import ExtendedColour
from pixy.TrueColour import TrueColour
from pixy.helpers import black, red, green, yellow, blue, magenta, cyan, white

__all__ = [
    "pring", "foreground", "background", "decorators", "Font", "EscapeSequence",
    "ExtendedColour", "TrueColour", "black", "red", "green", "yellow", "blue",
    "magenta", "cyan", "white"
]



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.EscapeSequence import EscapeSequence


class Font(EscapeSequence):

    __FONT_START_CODE = 11
    __FONT_END_CODE = 19

    def __new__(self, font_number: int):

        number_of_fonts = self.__FONT_END_CODE - self.__FONT_START_CODE

        if font_number < 0 or font_number > number_of_fonts:
            raise ValueError(
                "font must be in-between 0 and %d" % number_of_fonts
            )

        return super().__new__(self, self.__FONT_START_CODE + font_number)



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.pring import pring
from pixy.colours import foreground


def black(text: str) -> str:
    return pring(text, foreground.black)


def red(text: str) -> str:
    return pring(text, foreground.red)


def green(text: str) -> str:
    return pring(text, foreground.green)


def yellow(text: str) -> str:
    return pring(text, foreground.yellow)


def blue(text: str) -> str:
    return pring(text, foreground.blue)


def magenta(text: str) -> str:
    return pring(text, foreground.magenta)


def cyan(text: str) -> str:
    return pring(text, foreground.cyan)


def white(text: str) -> str:
    return pring(text, foreground.white)



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.EscapeSequence import EscapeSequence


reset = EscapeSequence(0)


def pring(text: str, style: EscapeSequence = EscapeSequence()) -> str:

    if not isinstance(text, str):
        raise ValueError("text must be of type str")

    if not isinstance(style, EscapeSequence):
        raise ValueError(
            "style must be of type EscapeSequence (or sub-class of)."
        )

    sections = [section for section in text.split(reset.code) if section]

    return style.code + (reset.code + style.code).join(sections) + reset.code



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.EscapeSequence import EscapeSequence
from pixy.colours import constants


class TrueColour(EscapeSequence):

    __MAX_VALUE = 0xFF
    __RGB_SUB_CODE = 2

    def __new__(self, red: int, green: int,
                blue: int, background: bool = False):

        if (min(red, green, blue) < 0 or
                max(red, green, blue) > self.__MAX_VALUE):

            raise ValueError(
                "RGB values must be in-between 0 and %d" % self.__MAX_VALUE
            )

        if background:
            type_code = constants.BACKGROUND_CODE
        else:
            type_code = constants.FOREGROUND_CODE

        return super().__new__(
            self,
            type_code,
            self.__RGB_SUB_CODE,
            red,
            green,
            blue
        )



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

BLACK = 0
RED = 1
GREEN = 2
YELLOW = 3
BLUE = 4
MAGENTA = 5
CYAN = 6
WHITE = 7

FOREGROUND_START_CODE = 30
BACKGROUND_START_CODE = 40

FOREGROUND_CODE = 38
BACKGROUND_CODE = 48



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.EscapeSequence import EscapeSequence
from pixy.colours import constants


class ExtendedColour(EscapeSequence):

    def __new__(self, colour_code: int, background: bool = False):

        if colour_code < 0 or colour_code > 255:
            raise ValueError("colour_code must be in-between 0 and 255")

        if background:
            type_code = constants.BACKGROUND_CODE
        else:
            type_code = constants.FOREGROUND_CODE

        return super().__new__(self, type_code, 5, colour_code)



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

class EscapeSequence(tuple):

    __ESCAPE_CHARACTER = '\x1b'

    def __new__(self, *codes):

        for code in codes:
            if not isinstance(code, int):
                raise TypeError("All codes must be integers.")

        return super().__new__(self, codes)

    def __add__(self, other):
        return EscapeSequence(*super().__add__(other))

    def __str__(self):
        return self.code

    @property
    def code(self):
        return "%s[%sm" % (
            self.__ESCAPE_CHARACTER,
            ";".join(str(code) for code in self)
        )



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from setuptools import setup

import pixy.metadata

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="pixy",
    author="Kian Cross",
    author_email="kian@kiancross.co.uk",
    url="https://github.com/kiancross/pixy/",
    version=pixy.metadata.version,
    licence="Apache-2.0",
    packages=["pixy", "pixy.colours"],
    description="Add colour and style to terminal text",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires="~=3.5",

    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ]
)



#
# Copyright 2020 Kian Cross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from pixy.EscapeSequence import EscapeSequence

bold = EscapeSequence(1)
faint = EscapeSequence(2)
italic = EscapeSequence(3)
underline = EscapeSequence(4)
slow_blink = EscapeSequence(5)
rapid_blink = EscapeSequence(6)
invert = EscapeSequence(7)
conceal = EscapeSequence(8)
strike = EscapeSequence(9)
fraktur = EscapeSequence(20)
double_underline = EscapeSequence(21)
framed = EscapeSequence(51)
encircled = EscapeSequence(52)
overlined = EscapeSequence(53)
